/**
* High Availability Service (HA) for JOnAS
*
* Copyright (C) 2007,2008 Bull S.A.S.
* Copyright (C) 2006 Distributed Systems Lab.
* Universidad Polit??cnica de Madrid (Spain)
* Contact: http://lsd.ls.fi.upm.es/lsd
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: HAServerInterceptor.java 1965 2008-07-29 15:40:02Z loris $
* --------------------------------------------------------------------------
*/

package org.ow2.cmi.ha.interceptor.jrmp;

import java.io.IOException;
import java.util.Stack;

import org.ow2.carol.rmi.interceptor.api.JServerRequestInfo;
import org.ow2.carol.rmi.interceptor.spi.JServerRequestInterceptor;
import org.ow2.cmi.ha.RequestId;
import org.ow2.cmi.ha.interceptor.HAContext;
import org.ow2.cmi.ha.interceptor.HACurrent;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class represents the server side of the interceptor mechanism
 * implemented to transfer replication related information between client
 * and server sides
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 */
public class HAServerInterceptor implements JServerRequestInterceptor {

    /**
     * Interceptor name
     */
    private String interceptorName = "HAServerInterceptor";

    /**
     * Logger for traces
     */
    private static Log logger = LogFactory.getLog(HAServerInterceptor.class);

    /**
     * Constructor
     */
    public HAServerInterceptor() {
    }

    /**
     * Get the name of this interceptor
     * @return name
     */
    public String name() {
        return interceptorName;
    }

    /**
     * Analyzes the HAContext received and constructs the required
     * context on the HA current context (the thread associated class) on
     * the server side.
     */
    public void receiveRequest(final JServerRequestInfo jri) throws IOException {
        // Get the HAContext from the received info
        HAServiceContext receivedCtx = (HAServiceContext) jri.getRequestServiceContext(HAServiceContext.HA_CTX_ID);

        if(receivedCtx == null) {
            return;
        }

        logger.debug(receivedCtx.toString());

        // Get the current Thread/HAContext association class
        HACurrent current = HACurrent.getHACurrent();

        HAContext haContext = (HAContext) receivedCtx.getContextData();

        Stack<RequestId> requests = haContext.getRequests();
        current.setRequests(requests);

        boolean onFailover = haContext.isOnFailover();
        current.setOnFailover(onFailover);

        logger.debug("HACtx asocciated to Thread");
    }

    public void sendReply(final JServerRequestInfo jri) throws IOException {
        putHAContext(jri);
    }

    public void sendException(final JServerRequestInfo jri) throws IOException {
        putHAContext(jri);
    }

    public void sendOther(final JServerRequestInfo jri) throws IOException {
        putHAContext(jri);
    }

    private void putHAContext(final JServerRequestInfo jri) {

        // Get the current HA Context info associated to the thread
        HACurrent current = HACurrent.getHACurrent();

        HAContext haContext = new HAContext();

        Stack<RequestId> requests = current.getRequests();
        haContext.setRequests(requests);

        logger.debug("newHACtx: " + haContext);

        // Create the new HAContext to propagate between interceptors
        HAServiceContext haServiceContext = new HAServiceContext(haContext);

        // Finally, include the new HAContext created to be propagated through
        // the interceptor
        jri.addReplyServiceContext(haServiceContext);
    }
}
